
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _EXTERN_C_
#ifdef __cplusplus
#define _EXTERN_C_ extern "C"
#else /* __cplusplus */
#define _EXTERN_C_
#endif /* __cplusplus */
#endif /* _EXTERN_C_ */




#if (defined(PIC) || defined(__PIC__))
/* For shared libraries, declare these weak and figure out which one was linked
   based on which init wrapper was called.  See mpi_init wrappers.  */
#pragma weak pmpi_init
#pragma weak PMPI_INIT
#pragma weak pmpi_init_
#pragma weak pmpi_init__
#pragma weak pmpi_init_thread
#pragma weak PMPI_INIT_THREAD
#pragma weak pmpi_init_thread_
#pragma weak pmpi_init_thread__
#endif /* PIC */

_EXTERN_C_ void pmpi_init(MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init__(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT_THREAD(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread_(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread__(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);


/* Macros to enable and disable compiler warnings for deprecated functions.
 * These macros will be used to silent warnings about deprecated MPI functions,
 * as these should be wrapped, even if they are deprecated.
 *
 * Note: The macros support GCC and clang compilers only. For other compilers
 *       just add similar macros for your compiler.
 */
#ifndef WRAP_MPI_CALL_PREFIX
#if defined(__clang__)
#define WRAP_MPI_CALL_PREFIX        \
  _Pragma("clang diagnostic push"); \
  _Pragma("clang diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("clang diagnostic pop");
#elif defined(__INTEL_COMPILER)
#define WRAP_MPI_CALL_PREFIX \
  _Pragma("warning push");   \
  _Pragma("warning(disable:1786)");
#define WRAP_MPI_CALL_POSTFIX _Pragma("warning pop")
#elif (defined(__GNUC__) && \
  ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4))
#define  WRAP_MPI_CALL_PREFIX     \
  _Pragma("GCC diagnostic push"); \
  _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("GCC diagnostic pop");
#else
#define WRAP_MPI_CALL_PREFIX
#define WRAP_MPI_CALL_POSTFIX
#endif
#endif

/* This file is part of P^nMPI.
 *
 * Copyright (c)
 *  2008-2019 Lawrence Livermore National Laboratories, United States of America
 *  2011-2016 ZIH, Technische Universitaet Dresden, Federal Republic of Germany
 *  2013-2019 RWTH Aachen University, Federal Republic of Germany
 *
 *
 * P^nMPI is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation version 2.1 dated February 1999.
 *
 * P^nMPI is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with P^nMPI; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   51 Franklin St, Fifth Floor
 *   Boston, MA 02110, USA
 *
 *
 * Written by Martin Schulz, schulzm@llnl.gov.
 *
 * LLNL-CODE-402774
 */

#ifndef PNMPI_NEWSTACK_H
#define PNMPI_NEWSTACK_H


#include <pnmpi/service.h>


#ifdef  __cplusplus
extern "C" {
#endif


int PMPI_Abort_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int errorcode);

int PMPI_Accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int PMPI_Add_error_class_NewStack(PNMPI_modHandle_t stack, int *errorclass);

int PMPI_Add_error_code_NewStack(PNMPI_modHandle_t stack, int errorclass, int *errorcode);

int PMPI_Add_error_string_NewStack(PNMPI_modHandle_t stack, int errorcode, const char *string);

int PMPI_Address_NewStack(PNMPI_modHandle_t stack, void *location, MPI_Aint *address);

int PMPI_Allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Alloc_mem_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, MPI_Info info, void *baseptr);

int PMPI_Allreduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int PMPI_Alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int PMPI_Attr_delete_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval);

int PMPI_Attr_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval, void *attribute_val, int *flag);

int PMPI_Attr_put_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval, void *attribute_val);

int PMPI_Barrier_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm);

int PMPI_Bcast_NewStack(PNMPI_modHandle_t stack, void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

int PMPI_Bsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int PMPI_Bsend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Buffer_attach_NewStack(PNMPI_modHandle_t stack, void *buffer, int size);

int PMPI_Buffer_detach_NewStack(PNMPI_modHandle_t stack, void *buffer, int *size);

int PMPI_Cancel_NewStack(PNMPI_modHandle_t stack, MPI_Request *request);

int PMPI_Cart_coords_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int maxdims, int coords[]);

int PMPI_Cart_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);

int PMPI_Cart_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);

int PMPI_Cart_map_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);

int PMPI_Cart_rank_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const int coords[], int *rank);

int PMPI_Cart_shift_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);

int PMPI_Cart_sub_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);

int PMPI_Cartdim_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *ndims);

int PMPI_Close_port_NewStack(PNMPI_modHandle_t stack, const char *port_name);

int PMPI_Comm_accept_NewStack(PNMPI_modHandle_t stack, const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int PMPI_Comm_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int errorcode);

int PMPI_Comm_compare_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm1, MPI_Comm comm2, int *result);

int PMPI_Comm_connect_NewStack(PNMPI_modHandle_t stack, const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int PMPI_Comm_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);

int PMPI_Comm_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);

int PMPI_Comm_create_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);

int PMPI_Comm_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);

int PMPI_Comm_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval);

int PMPI_Comm_disconnect_NewStack(PNMPI_modHandle_t stack, MPI_Comm *comm);

int PMPI_Comm_dup_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Comm *newcomm);

int PMPI_Comm_dup_with_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);

int PMPI_Comm_free_NewStack(PNMPI_modHandle_t stack, MPI_Comm *comm);

int PMPI_Comm_free_keyval_NewStack(PNMPI_modHandle_t stack, int *comm_keyval);

int PMPI_Comm_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);

int PMPI_Comm_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler *erhandler);

int PMPI_Comm_get_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info *info_used);

int PMPI_Comm_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, char *comm_name, int *resultlen);

int PMPI_Comm_get_parent_NewStack(PNMPI_modHandle_t stack, MPI_Comm *parent);

int PMPI_Comm_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group *group);

int PMPI_Comm_idup_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);

int PMPI_Comm_join_NewStack(PNMPI_modHandle_t stack, int fd, MPI_Comm *intercomm);

int PMPI_Comm_rank_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *rank);

int PMPI_Comm_remote_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group *group);

int PMPI_Comm_remote_size_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *size);

int PMPI_Comm_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval, void *attribute_val);

int PMPI_Comm_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler errhandler);

int PMPI_Comm_set_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info info);

int PMPI_Comm_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const char *comm_name);

int PMPI_Comm_size_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *size);

int PMPI_Comm_split_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int color, int key, MPI_Comm *newcomm);

int PMPI_Comm_split_type_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);

int PMPI_Comm_test_inter_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *flag);

int PMPI_Compare_and_swap_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);

int PMPI_Dims_create_NewStack(PNMPI_modHandle_t stack, int nnodes, int ndims, int dims[]);

int PMPI_Dist_graph_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);

int PMPI_Dist_graph_create_adjacent_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);

int PMPI_Dist_graph_neighbors_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);

int PMPI_Dist_graph_neighbors_count_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);

int PMPI_Errhandler_create_NewStack(PNMPI_modHandle_t stack, MPI_Handler_function *function, MPI_Errhandler *errhandler);

int PMPI_Errhandler_free_NewStack(PNMPI_modHandle_t stack, MPI_Errhandler *errhandler);

int PMPI_Errhandler_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler *errhandler);

int PMPI_Errhandler_set_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler errhandler);

int PMPI_Error_class_NewStack(PNMPI_modHandle_t stack, int errorcode, int *errorclass);

int PMPI_Error_string_NewStack(PNMPI_modHandle_t stack, int errorcode, char *string, int *resultlen);

int PMPI_Exscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int PMPI_Fetch_and_op_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);

int PMPI_File_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int errorcode);

int PMPI_File_close_NewStack(PNMPI_modHandle_t stack, MPI_File *fh);

int PMPI_File_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);

int PMPI_File_delete_NewStack(PNMPI_modHandle_t stack, const char *filename, MPI_Info info);

int PMPI_File_get_amode_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int *amode);

int PMPI_File_get_atomicity_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int *flag);

int PMPI_File_get_byte_offset_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, MPI_Offset *disp);

int PMPI_File_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File file, MPI_Errhandler *errhandler);

int PMPI_File_get_group_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Group *group);

int PMPI_File_get_info_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Info *info_used);

int PMPI_File_get_position_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *offset);

int PMPI_File_get_position_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *offset);

int PMPI_File_get_size_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *size);

int PMPI_File_get_type_extent_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);

int PMPI_File_get_view_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);

int PMPI_File_iread_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int PMPI_File_iread_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int PMPI_File_iread_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int PMPI_File_iwrite_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int PMPI_File_iwrite_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int PMPI_File_iwrite_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int PMPI_File_open_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);

int PMPI_File_preallocate_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset size);

int PMPI_File_read_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_read_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_read_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int PMPI_File_read_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status);

int PMPI_File_read_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_read_at_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_read_at_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);

int PMPI_File_read_at_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status);

int PMPI_File_read_ordered_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_read_ordered_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int PMPI_File_read_ordered_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status);

int PMPI_File_read_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_seek_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, int whence);

int PMPI_File_seek_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, int whence);

int PMPI_File_set_atomicity_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int flag);

int PMPI_File_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File file, MPI_Errhandler errhandler);

int PMPI_File_set_info_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Info info);

int PMPI_File_set_size_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset size);

int PMPI_File_set_view_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);

int PMPI_File_sync_NewStack(PNMPI_modHandle_t stack, MPI_File fh);

int PMPI_File_write_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_write_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_write_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int PMPI_File_write_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status);

int PMPI_File_write_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_write_at_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_write_at_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);

int PMPI_File_write_at_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status);

int PMPI_File_write_ordered_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_File_write_ordered_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int PMPI_File_write_ordered_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status);

int PMPI_File_write_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int PMPI_Finalize_NewStack(PNMPI_modHandle_t stack);

int PMPI_Finalized_NewStack(PNMPI_modHandle_t stack, int *flag);

int PMPI_Free_mem_NewStack(PNMPI_modHandle_t stack, void *base);

int PMPI_Gather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int PMPI_Gatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);

int PMPI_Get_NewStack(PNMPI_modHandle_t stack, void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int PMPI_Get_accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int PMPI_Get_address_NewStack(PNMPI_modHandle_t stack, const void *location, MPI_Aint *address);

int PMPI_Get_count_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, int *count);

int PMPI_Get_elements_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, int *count);

int PMPI_Get_elements_x_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);

int PMPI_Get_library_version_NewStack(PNMPI_modHandle_t stack, char *version, int *resultlen);

int PMPI_Get_processor_name_NewStack(PNMPI_modHandle_t stack, char *name, int *resultlen);

int PMPI_Get_version_NewStack(PNMPI_modHandle_t stack, int *version, int *subversion);

int PMPI_Graph_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);

int PMPI_Graph_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);

int PMPI_Graph_map_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);

int PMPI_Graph_neighbors_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);

int PMPI_Graph_neighbors_count_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int *nneighbors);

int PMPI_Graphdims_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *nnodes, int *nedges);

int PMPI_Grequest_complete_NewStack(PNMPI_modHandle_t stack, MPI_Request request);

int PMPI_Grequest_start_NewStack(PNMPI_modHandle_t stack, MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);

int PMPI_Group_compare_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, int *result);

int PMPI_Group_difference_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int PMPI_Group_excl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int PMPI_Group_free_NewStack(PNMPI_modHandle_t stack, MPI_Group *group);

int PMPI_Group_incl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int PMPI_Group_intersection_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int PMPI_Group_range_excl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int PMPI_Group_range_incl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int PMPI_Group_rank_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int *rank);

int PMPI_Group_size_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int *size);

int PMPI_Group_translate_ranks_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);

int PMPI_Group_union_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int PMPI_Iallgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Iallgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Iallreduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int PMPI_Ialltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Ialltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Ialltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int PMPI_Ibarrier_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Request *request);

int PMPI_Ibcast_NewStack(PNMPI_modHandle_t stack, void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);

int PMPI_Ibsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Iexscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int PMPI_Igather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int PMPI_Igatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int PMPI_Improbe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);

int PMPI_Imrecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);

int PMPI_Ineighbor_allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Ineighbor_allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Ineighbor_alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Ineighbor_alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int PMPI_Ineighbor_alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int PMPI_Info_create_NewStack(PNMPI_modHandle_t stack, MPI_Info *info);

int PMPI_Info_delete_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key);

int PMPI_Info_dup_NewStack(PNMPI_modHandle_t stack, MPI_Info info, MPI_Info *newinfo);

int PMPI_Info_free_NewStack(PNMPI_modHandle_t stack, MPI_Info *info);

int PMPI_Info_get_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, int valuelen, char *value, int *flag);

int PMPI_Info_get_nkeys_NewStack(PNMPI_modHandle_t stack, MPI_Info info, int *nkeys);

int PMPI_Info_get_nthkey_NewStack(PNMPI_modHandle_t stack, MPI_Info info, int n, char *key);

int PMPI_Info_get_valuelen_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, int *valuelen, int *flag);

int PMPI_Info_set_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, const char *value);

int PMPI_Init_NewStack(PNMPI_modHandle_t stack, int *argc, char ***argv);

int PMPI_Init_thread_NewStack(PNMPI_modHandle_t stack, int *argc, char ***argv, int required, int *provided);

int PMPI_Initialized_NewStack(PNMPI_modHandle_t stack, int *flag);

int PMPI_Intercomm_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);

int PMPI_Intercomm_merge_NewStack(PNMPI_modHandle_t stack, MPI_Comm intercomm, int high, MPI_Comm *newintercomm);

int PMPI_Iprobe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);

int PMPI_Irecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Ireduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);

int PMPI_Ireduce_scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int PMPI_Ireduce_scatter_block_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int PMPI_Irsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Is_thread_main_NewStack(PNMPI_modHandle_t stack, int *flag);

int PMPI_Iscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int PMPI_Iscatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int PMPI_Iscatterv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int PMPI_Isend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Issend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Keyval_create_NewStack(PNMPI_modHandle_t stack, MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);

int PMPI_Keyval_free_NewStack(PNMPI_modHandle_t stack, int *keyval);

int PMPI_Lookup_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, char *port_name);

int PMPI_Mprobe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);

int PMPI_Mrecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);

int PMPI_Neighbor_allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Neighbor_allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Neighbor_alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Neighbor_alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int PMPI_Neighbor_alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int PMPI_Op_commutative_NewStack(PNMPI_modHandle_t stack, MPI_Op op, int *commute);

int PMPI_Op_create_NewStack(PNMPI_modHandle_t stack, MPI_User_function *function, int commute, MPI_Op *op);

int PMPI_Op_free_NewStack(PNMPI_modHandle_t stack, MPI_Op *op);

int PMPI_Open_port_NewStack(PNMPI_modHandle_t stack, MPI_Info info, char *port_name);

int PMPI_Pack_NewStack(PNMPI_modHandle_t stack, const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);

int PMPI_Pack_external_NewStack(PNMPI_modHandle_t stack, const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);

int PMPI_Pack_external_size_NewStack(PNMPI_modHandle_t stack, const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);

int PMPI_Pack_size_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);

int PMPI_Probe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, MPI_Status *status);

int PMPI_Publish_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, const char *port_name);

int PMPI_Put_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int PMPI_Query_thread_NewStack(PNMPI_modHandle_t stack, int *provided);

int PMPI_Raccumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int PMPI_Recv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);

int PMPI_Recv_init_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Reduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);

int PMPI_Reduce_local_NewStack(PNMPI_modHandle_t stack, const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);

int PMPI_Reduce_scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int PMPI_Reduce_scatter_block_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int PMPI_Register_datarep_NewStack(PNMPI_modHandle_t stack, const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);

int PMPI_Request_free_NewStack(PNMPI_modHandle_t stack, MPI_Request *request);

int PMPI_Request_get_status_NewStack(PNMPI_modHandle_t stack, MPI_Request request, int *flag, MPI_Status *status);

int PMPI_Rget_NewStack(PNMPI_modHandle_t stack, void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int PMPI_Rget_accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int PMPI_Rput_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int PMPI_Rsend_NewStack(PNMPI_modHandle_t stack, const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int PMPI_Rsend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Scan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int PMPI_Scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int PMPI_Scatterv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int PMPI_Send_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int PMPI_Send_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Sendrecv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int PMPI_Sendrecv_replace_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int PMPI_Ssend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int PMPI_Ssend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int PMPI_Start_NewStack(PNMPI_modHandle_t stack, MPI_Request *request);

int PMPI_Startall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[]);

int PMPI_Status_set_cancelled_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, int flag);

int PMPI_Status_set_elements_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, MPI_Datatype datatype, int count);

int PMPI_Status_set_elements_x_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, MPI_Datatype datatype, MPI_Count count);

int PMPI_Test_NewStack(PNMPI_modHandle_t stack, MPI_Request *request, int *flag, MPI_Status *status);

int PMPI_Test_cancelled_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, int *flag);

int PMPI_Testall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);

int PMPI_Testany_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);

int PMPI_Testsome_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int PMPI_Topo_test_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *status);

int PMPI_Type_commit_NewStack(PNMPI_modHandle_t stack, MPI_Datatype *type);

int PMPI_Type_contiguous_NewStack(PNMPI_modHandle_t stack, int count, MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_create_darray_NewStack(PNMPI_modHandle_t stack, int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_create_f90_complex_NewStack(PNMPI_modHandle_t stack, int p, int r, MPI_Datatype *newtype);

int PMPI_Type_create_f90_integer_NewStack(PNMPI_modHandle_t stack, int r, MPI_Datatype *newtype);

int PMPI_Type_create_f90_real_NewStack(PNMPI_modHandle_t stack, int p, int r, MPI_Datatype *newtype);

int PMPI_Type_create_hindexed_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_create_hindexed_block_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_create_hvector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_create_indexed_block_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);

int PMPI_Type_create_resized_NewStack(PNMPI_modHandle_t stack, MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);

int PMPI_Type_create_struct_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int PMPI_Type_create_subarray_NewStack(PNMPI_modHandle_t stack, int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval);

int PMPI_Type_dup_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Datatype *newtype);

int PMPI_Type_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *extent);

int PMPI_Type_free_NewStack(PNMPI_modHandle_t stack, MPI_Datatype *type);

int PMPI_Type_free_keyval_NewStack(PNMPI_modHandle_t stack, int *type_keyval);

int PMPI_Type_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);

int PMPI_Type_get_contents_NewStack(PNMPI_modHandle_t stack, MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);

int PMPI_Type_get_envelope_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);

int PMPI_Type_get_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);

int PMPI_Type_get_extent_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);

int PMPI_Type_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, char *type_name, int *resultlen);

int PMPI_Type_get_true_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);

int PMPI_Type_get_true_extent_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);

int PMPI_Type_hindexed_NewStack(PNMPI_modHandle_t stack, int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_hvector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_indexed_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Type_lb_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *lb);

int PMPI_Type_match_size_NewStack(PNMPI_modHandle_t stack, int typeclass, int size, MPI_Datatype *type);

int PMPI_Type_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval, void *attr_val);

int PMPI_Type_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, const char *type_name);

int PMPI_Type_size_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int *size);

int PMPI_Type_size_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Count *size);

int PMPI_Type_struct_NewStack(PNMPI_modHandle_t stack, int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int PMPI_Type_ub_NewStack(PNMPI_modHandle_t stack, MPI_Datatype mtype, MPI_Aint *ub);

int PMPI_Type_vector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int PMPI_Unpack_NewStack(PNMPI_modHandle_t stack, const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);

int PMPI_Unpack_external_NewStack(PNMPI_modHandle_t stack, const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);

int PMPI_Unpublish_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, const char *port_name);

int PMPI_Wait_NewStack(PNMPI_modHandle_t stack, MPI_Request *request, MPI_Status *status);

int PMPI_Waitall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);

int PMPI_Waitany_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);

int PMPI_Waitsome_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int PMPI_Win_allocate_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int PMPI_Win_allocate_shared_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int PMPI_Win_attach_NewStack(PNMPI_modHandle_t stack, MPI_Win win, void *base, MPI_Aint size);

int PMPI_Win_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int errorcode);

int PMPI_Win_complete_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int PMPI_Win_create_NewStack(PNMPI_modHandle_t stack, void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);

int PMPI_Win_create_dynamic_NewStack(PNMPI_modHandle_t stack, MPI_Info info, MPI_Comm comm, MPI_Win *win);

int PMPI_Win_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);

int PMPI_Win_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);

int PMPI_Win_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval);

int PMPI_Win_detach_NewStack(PNMPI_modHandle_t stack, MPI_Win win, const void *base);

int PMPI_Win_fence_NewStack(PNMPI_modHandle_t stack, int assert, MPI_Win win);

int PMPI_Win_flush_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win);

int PMPI_Win_flush_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int PMPI_Win_flush_local_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win);

int PMPI_Win_flush_local_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int PMPI_Win_free_NewStack(PNMPI_modHandle_t stack, MPI_Win *win);

int PMPI_Win_free_keyval_NewStack(PNMPI_modHandle_t stack, int *win_keyval);

int PMPI_Win_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval, void *attribute_val, int *flag);

int PMPI_Win_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Errhandler *errhandler);

int PMPI_Win_get_group_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Group *group);

int PMPI_Win_get_info_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Info *info_used);

int PMPI_Win_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Win win, char *win_name, int *resultlen);

int PMPI_Win_lock_NewStack(PNMPI_modHandle_t stack, int lock_type, int rank, int assert, MPI_Win win);

int PMPI_Win_lock_all_NewStack(PNMPI_modHandle_t stack, int assert, MPI_Win win);

int PMPI_Win_post_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int assert, MPI_Win win);

int PMPI_Win_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval, void *attribute_val);

int PMPI_Win_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Errhandler errhandler);

int PMPI_Win_set_info_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Info info);

int PMPI_Win_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Win win, const char *win_name);

int PMPI_Win_shared_query_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);

int PMPI_Win_start_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int assert, MPI_Win win);

int PMPI_Win_sync_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int PMPI_Win_test_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int *flag);

int PMPI_Win_unlock_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win);

int PMPI_Win_unlock_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int PMPI_Win_wait_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

double PMPI_Wtick_NewStack(PNMPI_modHandle_t stack);

double PMPI_Wtime_NewStack(PNMPI_modHandle_t stack);



int XMPI_Abort_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int errorcode);

int XMPI_Accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int XMPI_Add_error_class_NewStack(PNMPI_modHandle_t stack, int *errorclass);

int XMPI_Add_error_code_NewStack(PNMPI_modHandle_t stack, int errorclass, int *errorcode);

int XMPI_Add_error_string_NewStack(PNMPI_modHandle_t stack, int errorcode, const char *string);

int XMPI_Address_NewStack(PNMPI_modHandle_t stack, void *location, MPI_Aint *address);

int XMPI_Allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Alloc_mem_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, MPI_Info info, void *baseptr);

int XMPI_Allreduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int XMPI_Attr_delete_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval);

int XMPI_Attr_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval, void *attribute_val, int *flag);

int XMPI_Attr_put_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval, void *attribute_val);

int XMPI_Barrier_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm);

int XMPI_Bcast_NewStack(PNMPI_modHandle_t stack, void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

int XMPI_Bsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Bsend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Buffer_attach_NewStack(PNMPI_modHandle_t stack, void *buffer, int size);

int XMPI_Buffer_detach_NewStack(PNMPI_modHandle_t stack, void *buffer, int *size);

int XMPI_Cancel_NewStack(PNMPI_modHandle_t stack, MPI_Request *request);

int XMPI_Cart_coords_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int maxdims, int coords[]);

int XMPI_Cart_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);

int XMPI_Cart_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);

int XMPI_Cart_map_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);

int XMPI_Cart_rank_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const int coords[], int *rank);

int XMPI_Cart_shift_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);

int XMPI_Cart_sub_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);

int XMPI_Cartdim_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *ndims);

int XMPI_Close_port_NewStack(PNMPI_modHandle_t stack, const char *port_name);

int XMPI_Comm_accept_NewStack(PNMPI_modHandle_t stack, const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int XMPI_Comm_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int errorcode);

int XMPI_Comm_compare_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm1, MPI_Comm comm2, int *result);

int XMPI_Comm_connect_NewStack(PNMPI_modHandle_t stack, const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int XMPI_Comm_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);

int XMPI_Comm_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);

int XMPI_Comm_create_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);

int XMPI_Comm_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);

int XMPI_Comm_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval);

int XMPI_Comm_disconnect_NewStack(PNMPI_modHandle_t stack, MPI_Comm *comm);

int XMPI_Comm_dup_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Comm *newcomm);

int XMPI_Comm_dup_with_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);

int XMPI_Comm_free_NewStack(PNMPI_modHandle_t stack, MPI_Comm *comm);

int XMPI_Comm_free_keyval_NewStack(PNMPI_modHandle_t stack, int *comm_keyval);

int XMPI_Comm_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);

int XMPI_Comm_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler *erhandler);

int XMPI_Comm_get_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info *info_used);

int XMPI_Comm_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, char *comm_name, int *resultlen);

int XMPI_Comm_get_parent_NewStack(PNMPI_modHandle_t stack, MPI_Comm *parent);

int XMPI_Comm_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group *group);

int XMPI_Comm_idup_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);

int XMPI_Comm_join_NewStack(PNMPI_modHandle_t stack, int fd, MPI_Comm *intercomm);

int XMPI_Comm_rank_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *rank);

int XMPI_Comm_remote_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group *group);

int XMPI_Comm_remote_size_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *size);

int XMPI_Comm_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval, void *attribute_val);

int XMPI_Comm_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler errhandler);

int XMPI_Comm_set_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info info);

int XMPI_Comm_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const char *comm_name);

int XMPI_Comm_size_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *size);

int XMPI_Comm_split_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int color, int key, MPI_Comm *newcomm);

int XMPI_Comm_split_type_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);

int XMPI_Comm_test_inter_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *flag);

int XMPI_Compare_and_swap_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);

int XMPI_Dims_create_NewStack(PNMPI_modHandle_t stack, int nnodes, int ndims, int dims[]);

int XMPI_Dist_graph_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);

int XMPI_Dist_graph_create_adjacent_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);

int XMPI_Dist_graph_neighbors_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);

int XMPI_Dist_graph_neighbors_count_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);

int XMPI_Errhandler_create_NewStack(PNMPI_modHandle_t stack, MPI_Handler_function *function, MPI_Errhandler *errhandler);

int XMPI_Errhandler_free_NewStack(PNMPI_modHandle_t stack, MPI_Errhandler *errhandler);

int XMPI_Errhandler_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler *errhandler);

int XMPI_Errhandler_set_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler errhandler);

int XMPI_Error_class_NewStack(PNMPI_modHandle_t stack, int errorcode, int *errorclass);

int XMPI_Error_string_NewStack(PNMPI_modHandle_t stack, int errorcode, char *string, int *resultlen);

int XMPI_Exscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Fetch_and_op_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);

int XMPI_File_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int errorcode);

int XMPI_File_close_NewStack(PNMPI_modHandle_t stack, MPI_File *fh);

int XMPI_File_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);

int XMPI_File_delete_NewStack(PNMPI_modHandle_t stack, const char *filename, MPI_Info info);

int XMPI_File_get_amode_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int *amode);

int XMPI_File_get_atomicity_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int *flag);

int XMPI_File_get_byte_offset_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, MPI_Offset *disp);

int XMPI_File_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File file, MPI_Errhandler *errhandler);

int XMPI_File_get_group_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Group *group);

int XMPI_File_get_info_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Info *info_used);

int XMPI_File_get_position_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *offset);

int XMPI_File_get_position_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *offset);

int XMPI_File_get_size_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *size);

int XMPI_File_get_type_extent_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);

int XMPI_File_get_view_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);

int XMPI_File_iread_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iread_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iread_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iwrite_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iwrite_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iwrite_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_open_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);

int XMPI_File_preallocate_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset size);

int XMPI_File_read_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int XMPI_File_read_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status);

int XMPI_File_read_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_at_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_at_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);

int XMPI_File_read_at_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status);

int XMPI_File_read_ordered_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_ordered_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int XMPI_File_read_ordered_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status);

int XMPI_File_read_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_seek_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, int whence);

int XMPI_File_seek_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, int whence);

int XMPI_File_set_atomicity_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int flag);

int XMPI_File_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File file, MPI_Errhandler errhandler);

int XMPI_File_set_info_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Info info);

int XMPI_File_set_size_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset size);

int XMPI_File_set_view_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);

int XMPI_File_sync_NewStack(PNMPI_modHandle_t stack, MPI_File fh);

int XMPI_File_write_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int XMPI_File_write_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status);

int XMPI_File_write_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_at_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_at_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);

int XMPI_File_write_at_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status);

int XMPI_File_write_ordered_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_ordered_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int XMPI_File_write_ordered_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status);

int XMPI_File_write_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_Finalize_NewStack(PNMPI_modHandle_t stack);

int XMPI_Finalized_NewStack(PNMPI_modHandle_t stack, int *flag);

int XMPI_Free_mem_NewStack(PNMPI_modHandle_t stack, void *base);

int XMPI_Gather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Gatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Get_NewStack(PNMPI_modHandle_t stack, void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int XMPI_Get_accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int XMPI_Get_address_NewStack(PNMPI_modHandle_t stack, const void *location, MPI_Aint *address);

int XMPI_Get_count_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, int *count);

int XMPI_Get_elements_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, int *count);

int XMPI_Get_elements_x_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);

int XMPI_Get_library_version_NewStack(PNMPI_modHandle_t stack, char *version, int *resultlen);

int XMPI_Get_processor_name_NewStack(PNMPI_modHandle_t stack, char *name, int *resultlen);

int XMPI_Get_version_NewStack(PNMPI_modHandle_t stack, int *version, int *subversion);

int XMPI_Graph_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);

int XMPI_Graph_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);

int XMPI_Graph_map_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);

int XMPI_Graph_neighbors_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);

int XMPI_Graph_neighbors_count_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int *nneighbors);

int XMPI_Graphdims_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *nnodes, int *nedges);

int XMPI_Grequest_complete_NewStack(PNMPI_modHandle_t stack, MPI_Request request);

int XMPI_Grequest_start_NewStack(PNMPI_modHandle_t stack, MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);

int XMPI_Group_compare_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, int *result);

int XMPI_Group_difference_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int XMPI_Group_excl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int XMPI_Group_free_NewStack(PNMPI_modHandle_t stack, MPI_Group *group);

int XMPI_Group_incl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int XMPI_Group_intersection_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int XMPI_Group_range_excl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int XMPI_Group_range_incl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int XMPI_Group_rank_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int *rank);

int XMPI_Group_size_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int *size);

int XMPI_Group_translate_ranks_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);

int XMPI_Group_union_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int XMPI_Iallgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Iallgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Iallreduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Ialltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ialltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ialltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int XMPI_Ibarrier_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Request *request);

int XMPI_Ibcast_NewStack(PNMPI_modHandle_t stack, void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Ibsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Iexscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Igather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Igatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Improbe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);

int XMPI_Imrecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);

int XMPI_Ineighbor_allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int XMPI_Info_create_NewStack(PNMPI_modHandle_t stack, MPI_Info *info);

int XMPI_Info_delete_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key);

int XMPI_Info_dup_NewStack(PNMPI_modHandle_t stack, MPI_Info info, MPI_Info *newinfo);

int XMPI_Info_free_NewStack(PNMPI_modHandle_t stack, MPI_Info *info);

int XMPI_Info_get_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, int valuelen, char *value, int *flag);

int XMPI_Info_get_nkeys_NewStack(PNMPI_modHandle_t stack, MPI_Info info, int *nkeys);

int XMPI_Info_get_nthkey_NewStack(PNMPI_modHandle_t stack, MPI_Info info, int n, char *key);

int XMPI_Info_get_valuelen_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, int *valuelen, int *flag);

int XMPI_Info_set_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, const char *value);

int XMPI_Init_NewStack(PNMPI_modHandle_t stack, int *argc, char ***argv);

int XMPI_Init_thread_NewStack(PNMPI_modHandle_t stack, int *argc, char ***argv, int required, int *provided);

int XMPI_Initialized_NewStack(PNMPI_modHandle_t stack, int *flag);

int XMPI_Intercomm_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);

int XMPI_Intercomm_merge_NewStack(PNMPI_modHandle_t stack, MPI_Comm intercomm, int high, MPI_Comm *newintercomm);

int XMPI_Iprobe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);

int XMPI_Irecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Ireduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Ireduce_scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Ireduce_scatter_block_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Irsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Is_thread_main_NewStack(PNMPI_modHandle_t stack, int *flag);

int XMPI_Iscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Iscatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Iscatterv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Isend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Issend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Keyval_create_NewStack(PNMPI_modHandle_t stack, MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);

int XMPI_Keyval_free_NewStack(PNMPI_modHandle_t stack, int *keyval);

int XMPI_Lookup_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, char *port_name);

int XMPI_Mprobe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);

int XMPI_Mrecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);

int XMPI_Neighbor_allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int XMPI_Op_commutative_NewStack(PNMPI_modHandle_t stack, MPI_Op op, int *commute);

int XMPI_Op_create_NewStack(PNMPI_modHandle_t stack, MPI_User_function *function, int commute, MPI_Op *op);

int XMPI_Op_free_NewStack(PNMPI_modHandle_t stack, MPI_Op *op);

int XMPI_Open_port_NewStack(PNMPI_modHandle_t stack, MPI_Info info, char *port_name);

int XMPI_Pack_NewStack(PNMPI_modHandle_t stack, const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);

int XMPI_Pack_external_NewStack(PNMPI_modHandle_t stack, const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);

int XMPI_Pack_external_size_NewStack(PNMPI_modHandle_t stack, const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);

int XMPI_Pack_size_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);

int XMPI_Probe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, MPI_Status *status);

int XMPI_Publish_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, const char *port_name);

int XMPI_Put_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int XMPI_Query_thread_NewStack(PNMPI_modHandle_t stack, int *provided);

int XMPI_Raccumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int XMPI_Recv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);

int XMPI_Recv_init_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Reduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);

int XMPI_Reduce_local_NewStack(PNMPI_modHandle_t stack, const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);

int XMPI_Reduce_scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Reduce_scatter_block_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Register_datarep_NewStack(PNMPI_modHandle_t stack, const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);

int XMPI_Request_free_NewStack(PNMPI_modHandle_t stack, MPI_Request *request);

int XMPI_Request_get_status_NewStack(PNMPI_modHandle_t stack, MPI_Request request, int *flag, MPI_Status *status);

int XMPI_Rget_NewStack(PNMPI_modHandle_t stack, void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int XMPI_Rget_accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int XMPI_Rput_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int XMPI_Rsend_NewStack(PNMPI_modHandle_t stack, const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Rsend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Scan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Scatterv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Send_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Send_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Sendrecv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int XMPI_Sendrecv_replace_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int XMPI_Ssend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Ssend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Start_NewStack(PNMPI_modHandle_t stack, MPI_Request *request);

int XMPI_Startall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[]);

int XMPI_Status_set_cancelled_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, int flag);

int XMPI_Status_set_elements_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, MPI_Datatype datatype, int count);

int XMPI_Status_set_elements_x_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, MPI_Datatype datatype, MPI_Count count);

int XMPI_Test_NewStack(PNMPI_modHandle_t stack, MPI_Request *request, int *flag, MPI_Status *status);

int XMPI_Test_cancelled_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, int *flag);

int XMPI_Testall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);

int XMPI_Testany_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);

int XMPI_Testsome_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int XMPI_Topo_test_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *status);

int XMPI_Type_commit_NewStack(PNMPI_modHandle_t stack, MPI_Datatype *type);

int XMPI_Type_contiguous_NewStack(PNMPI_modHandle_t stack, int count, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_darray_NewStack(PNMPI_modHandle_t stack, int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_f90_complex_NewStack(PNMPI_modHandle_t stack, int p, int r, MPI_Datatype *newtype);

int XMPI_Type_create_f90_integer_NewStack(PNMPI_modHandle_t stack, int r, MPI_Datatype *newtype);

int XMPI_Type_create_f90_real_NewStack(PNMPI_modHandle_t stack, int p, int r, MPI_Datatype *newtype);

int XMPI_Type_create_hindexed_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_hindexed_block_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_hvector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_indexed_block_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);

int XMPI_Type_create_resized_NewStack(PNMPI_modHandle_t stack, MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);

int XMPI_Type_create_struct_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int XMPI_Type_create_subarray_NewStack(PNMPI_modHandle_t stack, int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval);

int XMPI_Type_dup_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Datatype *newtype);

int XMPI_Type_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *extent);

int XMPI_Type_free_NewStack(PNMPI_modHandle_t stack, MPI_Datatype *type);

int XMPI_Type_free_keyval_NewStack(PNMPI_modHandle_t stack, int *type_keyval);

int XMPI_Type_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);

int XMPI_Type_get_contents_NewStack(PNMPI_modHandle_t stack, MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);

int XMPI_Type_get_envelope_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);

int XMPI_Type_get_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);

int XMPI_Type_get_extent_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);

int XMPI_Type_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, char *type_name, int *resultlen);

int XMPI_Type_get_true_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);

int XMPI_Type_get_true_extent_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);

int XMPI_Type_hindexed_NewStack(PNMPI_modHandle_t stack, int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_hvector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_indexed_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_lb_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *lb);

int XMPI_Type_match_size_NewStack(PNMPI_modHandle_t stack, int typeclass, int size, MPI_Datatype *type);

int XMPI_Type_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval, void *attr_val);

int XMPI_Type_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, const char *type_name);

int XMPI_Type_size_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int *size);

int XMPI_Type_size_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Count *size);

int XMPI_Type_struct_NewStack(PNMPI_modHandle_t stack, int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int XMPI_Type_ub_NewStack(PNMPI_modHandle_t stack, MPI_Datatype mtype, MPI_Aint *ub);

int XMPI_Type_vector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Unpack_NewStack(PNMPI_modHandle_t stack, const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);

int XMPI_Unpack_external_NewStack(PNMPI_modHandle_t stack, const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);

int XMPI_Unpublish_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, const char *port_name);

int XMPI_Wait_NewStack(PNMPI_modHandle_t stack, MPI_Request *request, MPI_Status *status);

int XMPI_Waitall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);

int XMPI_Waitany_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);

int XMPI_Waitsome_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int XMPI_Win_allocate_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int XMPI_Win_allocate_shared_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int XMPI_Win_attach_NewStack(PNMPI_modHandle_t stack, MPI_Win win, void *base, MPI_Aint size);

int XMPI_Win_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int errorcode);

int XMPI_Win_complete_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int XMPI_Win_create_NewStack(PNMPI_modHandle_t stack, void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);

int XMPI_Win_create_dynamic_NewStack(PNMPI_modHandle_t stack, MPI_Info info, MPI_Comm comm, MPI_Win *win);

int XMPI_Win_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);

int XMPI_Win_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);

int XMPI_Win_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval);

int XMPI_Win_detach_NewStack(PNMPI_modHandle_t stack, MPI_Win win, const void *base);

int XMPI_Win_fence_NewStack(PNMPI_modHandle_t stack, int assert, MPI_Win win);

int XMPI_Win_flush_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win);

int XMPI_Win_flush_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int XMPI_Win_flush_local_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win);

int XMPI_Win_flush_local_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int XMPI_Win_free_NewStack(PNMPI_modHandle_t stack, MPI_Win *win);

int XMPI_Win_free_keyval_NewStack(PNMPI_modHandle_t stack, int *win_keyval);

int XMPI_Win_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval, void *attribute_val, int *flag);

int XMPI_Win_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Errhandler *errhandler);

int XMPI_Win_get_group_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Group *group);

int XMPI_Win_get_info_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Info *info_used);

int XMPI_Win_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Win win, char *win_name, int *resultlen);

int XMPI_Win_lock_NewStack(PNMPI_modHandle_t stack, int lock_type, int rank, int assert, MPI_Win win);

int XMPI_Win_lock_all_NewStack(PNMPI_modHandle_t stack, int assert, MPI_Win win);

int XMPI_Win_post_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int assert, MPI_Win win);

int XMPI_Win_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval, void *attribute_val);

int XMPI_Win_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Errhandler errhandler);

int XMPI_Win_set_info_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Info info);

int XMPI_Win_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Win win, const char *win_name);

int XMPI_Win_shared_query_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);

int XMPI_Win_start_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int assert, MPI_Win win);

int XMPI_Win_sync_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int XMPI_Win_test_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int *flag);

int XMPI_Win_unlock_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win);

int XMPI_Win_unlock_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

int XMPI_Win_wait_NewStack(PNMPI_modHandle_t stack, MPI_Win win);

double XMPI_Wtick_NewStack(PNMPI_modHandle_t stack);

double XMPI_Wtime_NewStack(PNMPI_modHandle_t stack);


#ifdef  __cplusplus
}
#endif
#endif
