
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _EXTERN_C_
#ifdef __cplusplus
#define _EXTERN_C_ extern "C"
#else /* __cplusplus */
#define _EXTERN_C_
#endif /* __cplusplus */
#endif /* _EXTERN_C_ */




#if (defined(PIC) || defined(__PIC__))
/* For shared libraries, declare these weak and figure out which one was linked
   based on which init wrapper was called.  See mpi_init wrappers.  */
#pragma weak pmpi_init
#pragma weak PMPI_INIT
#pragma weak pmpi_init_
#pragma weak pmpi_init__
#pragma weak pmpi_init_thread
#pragma weak PMPI_INIT_THREAD
#pragma weak pmpi_init_thread_
#pragma weak pmpi_init_thread__
#endif /* PIC */

_EXTERN_C_ void pmpi_init(MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init__(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT_THREAD(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread_(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread__(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);


/* Macros to enable and disable compiler warnings for deprecated functions.
 * These macros will be used to silent warnings about deprecated MPI functions,
 * as these should be wrapped, even if they are deprecated.
 *
 * Note: The macros support GCC and clang compilers only. For other compilers
 *       just add similar macros for your compiler.
 */
#ifndef WRAP_MPI_CALL_PREFIX
#if defined(__clang__)
#define WRAP_MPI_CALL_PREFIX        \
  _Pragma("clang diagnostic push"); \
  _Pragma("clang diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("clang diagnostic pop");
#elif defined(__INTEL_COMPILER)
#define WRAP_MPI_CALL_PREFIX \
  _Pragma("warning push");   \
  _Pragma("warning(disable:1786)");
#define WRAP_MPI_CALL_POSTFIX _Pragma("warning pop")
#elif (defined(__GNUC__) && \
  ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4))
#define  WRAP_MPI_CALL_PREFIX     \
  _Pragma("GCC diagnostic push"); \
  _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("GCC diagnostic pop");
#else
#define WRAP_MPI_CALL_PREFIX
#define WRAP_MPI_CALL_POSTFIX
#endif
#endif

/* This file is part of P^nMPI.
 *
 * Copyright (c)
 *  2008-2019 Lawrence Livermore National Laboratories, United States of America
 *  2011-2016 ZIH, Technische Universitaet Dresden, Federal Republic of Germany
 *  2013-2019 RWTH Aachen University, Federal Republic of Germany
 *
 *
 * P^nMPI is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation version 2.1 dated February 1999.
 *
 * P^nMPI is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with P^nMPI; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   51 Franklin St, Fifth Floor
 *   Boston, MA 02110, USA
 *
 *
 * Written by Martin Schulz, schulzm@llnl.gov.
 *
 * LLNL-CODE-402774
 */

#ifndef PNMPI_XMPI_H
#define PNMPI_XMPI_H

#ifdef  __cplusplus
extern "C" {
#endif


int XMPI_Abort(MPI_Comm comm, int errorcode);

int XMPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int XMPI_Add_error_class(int *errorclass);

int XMPI_Add_error_code(int errorclass, int *errorcode);

int XMPI_Add_error_string(int errorcode, const char *string);

int XMPI_Address(void *location, MPI_Aint *address);

int XMPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr);

int XMPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int XMPI_Attr_delete(MPI_Comm comm, int keyval);

int XMPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag);

int XMPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val);

int XMPI_Barrier(MPI_Comm comm);

int XMPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

int XMPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Buffer_attach(void *buffer, int size);

int XMPI_Buffer_detach(void *buffer, int *size);

int XMPI_Cancel(MPI_Request *request);

int XMPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]);

int XMPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);

int XMPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);

int XMPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);

int XMPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank);

int XMPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);

int XMPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);

int XMPI_Cartdim_get(MPI_Comm comm, int *ndims);

int XMPI_Close_port(const char *port_name);

int XMPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int XMPI_Comm_call_errhandler(MPI_Comm comm, int errorcode);

int XMPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result);

int XMPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int XMPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);

int XMPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);

int XMPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);

int XMPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);

int XMPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval);

int XMPI_Comm_disconnect(MPI_Comm *comm);

int XMPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm);

int XMPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);

int XMPI_Comm_free(MPI_Comm *comm);

int XMPI_Comm_free_keyval(int *comm_keyval);

int XMPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);

int XMPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler);

int XMPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used);

int XMPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen);

int XMPI_Comm_get_parent(MPI_Comm *parent);

int XMPI_Comm_group(MPI_Comm comm, MPI_Group *group);

int XMPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);

int XMPI_Comm_join(int fd, MPI_Comm *intercomm);

int XMPI_Comm_rank(MPI_Comm comm, int *rank);

int XMPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group);

int XMPI_Comm_remote_size(MPI_Comm comm, int *size);

int XMPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val);

int XMPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler);

int XMPI_Comm_set_info(MPI_Comm comm, MPI_Info info);

int XMPI_Comm_set_name(MPI_Comm comm, const char *comm_name);

int XMPI_Comm_size(MPI_Comm comm, int *size);

int XMPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm);

int XMPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);

int XMPI_Comm_test_inter(MPI_Comm comm, int *flag);

int XMPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);

int XMPI_Dims_create(int nnodes, int ndims, int dims[]);

int XMPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);

int XMPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);

int XMPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);

int XMPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);

int XMPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler);

int XMPI_Errhandler_free(MPI_Errhandler *errhandler);

int XMPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler);

int XMPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler);

int XMPI_Error_class(int errorcode, int *errorclass);

int XMPI_Error_string(int errorcode, char *string, int *resultlen);

int XMPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);

int XMPI_File_call_errhandler(MPI_File fh, int errorcode);

int XMPI_File_close(MPI_File *fh);

int XMPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);

int XMPI_File_delete(const char *filename, MPI_Info info);

int XMPI_File_get_amode(MPI_File fh, int *amode);

int XMPI_File_get_atomicity(MPI_File fh, int *flag);

int XMPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp);

int XMPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler);

int XMPI_File_get_group(MPI_File fh, MPI_Group *group);

int XMPI_File_get_info(MPI_File fh, MPI_Info *info_used);

int XMPI_File_get_position(MPI_File fh, MPI_Offset *offset);

int XMPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset);

int XMPI_File_get_size(MPI_File fh, MPI_Offset *size);

int XMPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);

int XMPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);

int XMPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int XMPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);

int XMPI_File_preallocate(MPI_File fh, MPI_Offset size);

int XMPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int XMPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status);

int XMPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);

int XMPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status);

int XMPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int XMPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status);

int XMPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_seek(MPI_File fh, MPI_Offset offset, int whence);

int XMPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence);

int XMPI_File_set_atomicity(MPI_File fh, int flag);

int XMPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler);

int XMPI_File_set_info(MPI_File fh, MPI_Info info);

int XMPI_File_set_size(MPI_File fh, MPI_Offset size);

int XMPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);

int XMPI_File_sync(MPI_File fh);

int XMPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int XMPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status);

int XMPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);

int XMPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status);

int XMPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int XMPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status);

int XMPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int XMPI_Finalize();

int XMPI_Finalized(int *flag);

int XMPI_Free_mem(void *base);

int XMPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int XMPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int XMPI_Get_address(const void *location, MPI_Aint *address);

int XMPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count);

int XMPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count);

int XMPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);

int XMPI_Get_library_version(char *version, int *resultlen);

int XMPI_Get_processor_name(char *name, int *resultlen);

int XMPI_Get_version(int *version, int *subversion);

int XMPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);

int XMPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);

int XMPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);

int XMPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);

int XMPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors);

int XMPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges);

int XMPI_Grequest_complete(MPI_Request request);

int XMPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);

int XMPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result);

int XMPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int XMPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int XMPI_Group_free(MPI_Group *group);

int XMPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int XMPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int XMPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int XMPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int XMPI_Group_rank(MPI_Group group, int *rank);

int XMPI_Group_size(MPI_Group group, int *size);

int XMPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);

int XMPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int XMPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int XMPI_Ibarrier(MPI_Comm comm, MPI_Request *request);

int XMPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);

int XMPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);

int XMPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int XMPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int XMPI_Info_create(MPI_Info *info);

int XMPI_Info_delete(MPI_Info info, const char *key);

int XMPI_Info_dup(MPI_Info info, MPI_Info *newinfo);

int XMPI_Info_free(MPI_Info *info);

int XMPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag);

int XMPI_Info_get_nkeys(MPI_Info info, int *nkeys);

int XMPI_Info_get_nthkey(MPI_Info info, int n, char *key);

int XMPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag);

int XMPI_Info_set(MPI_Info info, const char *key, const char *value);

int XMPI_Init(int *argc, char ***argv);

int XMPI_Init_thread(int *argc, char ***argv, int required, int *provided);

int XMPI_Initialized(int *flag);

int XMPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);

int XMPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm);

int XMPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);

int XMPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Is_thread_main(int *flag);

int XMPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int XMPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int XMPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);

int XMPI_Keyval_free(int *keyval);

int XMPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name);

int XMPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);

int XMPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);

int XMPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int XMPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int XMPI_Op_commutative(MPI_Op op, int *commute);

int XMPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op);

int XMPI_Op_free(MPI_Op *op);

int XMPI_Open_port(MPI_Info info, char *port_name);

int XMPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);

int XMPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);

int XMPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);

int XMPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);

int XMPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status);

int XMPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name);

int XMPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int XMPI_Query_thread(int *provided);

int XMPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int XMPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);

int XMPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);

int XMPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);

int XMPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);

int XMPI_Request_free(MPI_Request *request);

int XMPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status);

int XMPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int XMPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int XMPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int XMPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int XMPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int XMPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int XMPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int XMPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int XMPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int XMPI_Start(MPI_Request *request);

int XMPI_Startall(int count, MPI_Request array_of_requests[]);

int XMPI_Status_set_cancelled(MPI_Status *status, int flag);

int XMPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count);

int XMPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count);

int XMPI_Test(MPI_Request *request, int *flag, MPI_Status *status);

int XMPI_Test_cancelled(const MPI_Status *status, int *flag);

int XMPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);

int XMPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);

int XMPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int XMPI_Topo_test(MPI_Comm comm, int *status);

int XMPI_Type_commit(MPI_Datatype *type);

int XMPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype);

int XMPI_Type_create_f90_integer(int r, MPI_Datatype *newtype);

int XMPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype);

int XMPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);

int XMPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);

int XMPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int XMPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_delete_attr(MPI_Datatype type, int type_keyval);

int XMPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype);

int XMPI_Type_extent(MPI_Datatype type, MPI_Aint *extent);

int XMPI_Type_free(MPI_Datatype *type);

int XMPI_Type_free_keyval(int *type_keyval);

int XMPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);

int XMPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);

int XMPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);

int XMPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);

int XMPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);

int XMPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen);

int XMPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);

int XMPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);

int XMPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Type_lb(MPI_Datatype type, MPI_Aint *lb);

int XMPI_Type_match_size(int typeclass, int size, MPI_Datatype *type);

int XMPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val);

int XMPI_Type_set_name(MPI_Datatype type, const char *type_name);

int XMPI_Type_size(MPI_Datatype type, int *size);

int XMPI_Type_size_x(MPI_Datatype type, MPI_Count *size);

int XMPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int XMPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub);

int XMPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int XMPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);

int XMPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);

int XMPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name);

int XMPI_Wait(MPI_Request *request, MPI_Status *status);

int XMPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);

int XMPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);

int XMPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int XMPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int XMPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int XMPI_Win_attach(MPI_Win win, void *base, MPI_Aint size);

int XMPI_Win_call_errhandler(MPI_Win win, int errorcode);

int XMPI_Win_complete(MPI_Win win);

int XMPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);

int XMPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win);

int XMPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);

int XMPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);

int XMPI_Win_delete_attr(MPI_Win win, int win_keyval);

int XMPI_Win_detach(MPI_Win win, const void *base);

int XMPI_Win_fence(int assert, MPI_Win win);

int XMPI_Win_flush(int rank, MPI_Win win);

int XMPI_Win_flush_all(MPI_Win win);

int XMPI_Win_flush_local(int rank, MPI_Win win);

int XMPI_Win_flush_local_all(MPI_Win win);

int XMPI_Win_free(MPI_Win *win);

int XMPI_Win_free_keyval(int *win_keyval);

int XMPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag);

int XMPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler);

int XMPI_Win_get_group(MPI_Win win, MPI_Group *group);

int XMPI_Win_get_info(MPI_Win win, MPI_Info *info_used);

int XMPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen);

int XMPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win);

int XMPI_Win_lock_all(int assert, MPI_Win win);

int XMPI_Win_post(MPI_Group group, int assert, MPI_Win win);

int XMPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val);

int XMPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler);

int XMPI_Win_set_info(MPI_Win win, MPI_Info info);

int XMPI_Win_set_name(MPI_Win win, const char *win_name);

int XMPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);

int XMPI_Win_start(MPI_Group group, int assert, MPI_Win win);

int XMPI_Win_sync(MPI_Win win);

int XMPI_Win_test(MPI_Win win, int *flag);

int XMPI_Win_unlock(int rank, MPI_Win win);

int XMPI_Win_unlock_all(MPI_Win win);

int XMPI_Win_wait(MPI_Win win);

double XMPI_Wtick();

double XMPI_Wtime();


#ifdef  __cplusplus
}
#endif

#endif
